************************************************
* Tue Jul 14 10:37:21 CDT 1987
*
************************************************
* BOOT
*
* BOOTLOADER FIRMWARE FOR 68HC11E9
*
* EQUATES FOR USE WITH INDEX OFFSET = $1000
*
*
PORTD   EQU     $08
DDRD    EQU     $09
TOC1    EQU     $16     [EXTRA STORAGE (POOR STYLE)]
SPCR    EQU     $28     (FOR DWOM BIT)
BAUD    EQU     $2B
SCCR1   EQU     $2C
SCCR2   EQU     $2D
SCSR    EQU     $2E
SCDAT   EQU     $2F
PPROG   EQU     $3B
TEST1   EQU     $3E
CONFIG  EQU     $3F
*
* MORE EQUATES
*
EEPSTR  EQU     $B600           START OF EEPROM
EEPEND  EQU     $B7FF           END OF EEPROM
*
********
* THIS BOOTSTRAP PROGRAM ALLOWS THE USER TO
* DOWNLOAD A PROGRAM OF 0 - 512 BYTES.
* THE PROGRAM MUST START AT $0000.
* EACH BYTE OF THE PROGRAM IS RECEIVED BY THE SCI.
* THE FIRST BYTE ESTABLISHES BAUD RATE.
* THE SECOND AND THIRD BYTES ARE THE NUMBER OF
* BEING SENT (0 - $200).
* THEN THE PROGRAM IS DOWNLOADED STARTING WITH
* THE $0000 BYTE AND WORKING UP TOWARD THE $01FF
* BYTE FOR THE TOTAL NUMBER OF BYTES SPECIFIED.
*
* THE TRANSMITTER WILL BE USED FOR THE PURPOSE
* OF COMMUNICATION TO THE OUTSIDE WORLD.
*
************************************************
        ORG     $BF40
*
BEGIN   EQU     *
* INIT STACK
        LDS     #$01FF
* INIT X REG FOR INDEXED ACCESS TO REGISTERS
        LDX     #$1000
********
* PUT PORT D IN WIRE OR MODE
        BSET    SPCR,X $20
* INIT SCI AND RESTART BAUD DIVIDER CHAIN
        LDD     #$A20C          DIV BY 16
        STAA    BAUD,X
* RECEIVER & TRANSMITTER ENABLED
        STAB    SCCR2,X
********
* TEST THE SECURITY BIT
        BRSET   CONFIG,X $08 NOSEC
************************************************
* WE ARE IN SECURITY MODE
*
* OUTPUT $FF ON TRANSMITTER
AGAIN   EQU     *
        LDAA    SCSR,X
        LDAA    #$FF
        STAA    SCDAT,X
* ACCA NOW IS SET FOR $FF
*
* ERASE EEPROM:
* SET ERASE AND EELAT BITS BEFORE USING "ERASE"
        LDAB    #$06
        STAB    PPROG,X
        STAB    EEPSTR          WRITE EEPROM LOCATION
        BSR     ERASE
* ACCB IS NOW SET FOR $06
*
********
* ERASE CYCLE IS COMPLETE
*
* IF THE EEPROM IS NOT ENABLED,
* WE CAN'T CHECK THAT THE EEPROM IS ERASED
        BRCLR   CONFIG,X $01 NOEE
* EEPROM IS ON,
* NOW CHECK THAT THE EEPROM IS ERASED
        LDY     #EEPSTR
LOOP    EQU     *
        CMPA    0,Y             (A = $FF)
* ANY UNERASED BYTE SENDS US BACK TO ERASE AGAIN
        BNE     AGAIN
        INY
        CPY     #EEPEND+1
        BNE     LOOP
NOEE    EQU     *
************************************************
* WRITE OVER ENTIRE RAM (EXCEPT LAST TWO BYTES
* WHICH ARE USED BY THE STACK)
*
ERAM    EQU     *
        PSHX
        LDX     #$0000
LOP1    EQU     *
        STAA    $00,X
        INX
        CPX     #$01FE
        BNE     LOP1
        PULX
************************************************
* NOW ERASE CONFIG REGISTER
*
ECONFG  EQU     *
* SET ERASE AND EELAT BITS
        STAB    PPROG,X         (B STILL = $06)
* WRITE CONFIG REGISTER LATCH IT FOR ERASURE
        STAB    CONFIG,X
        BSR     ERASE
********
* ERASE CYCLE IS COMPLETE
*
************************************************
* NON-SECURITY AND SECURITY MODES MEET HERE
*
NOSEC   EQU     *
* SEND BREAK TO SIGNAL START OF DOWNLOAD
        BSET    SCCR2,X $01
* CLEAR BREAK AS SOON AS START BIT IS DETECTED
        BRSET   PORTD,X $01 *
        BCLR    SCCR2,X $01     CLEAR BREAK
* WAIT FOR FIRST CHARACTER (USERS SEND $FF)
        BRCLR   SCSR,X $20 *    WAIT FOR RDRF
        LDAA    SCDAT,X         READ DATA
* IF DATA = $00 (BREAK OR $00), THEN JUMP TO EEPROM
        BNE     NOTZERO
        JMP     EEPSTR
NOTZERO EQU     *
* IF DATA = $FF, THEN /16 IS CORRECT BAUD
        CMPA    #$FF
        BEQ     BAUDOK
* ELSE CHANGE TO /104 (/13 & /8) 1200 @ 2MHZ
        BSET    BAUD,X $33
BAUDOK  EQU     *
* GET BYTE COUNT
        BRCLR   SCSR,X $20 *    WAIT FOR RDRF
        LDAA    SCDAT,X         LENGTH OF PROG (HI BYTE)
        ANDA    #$03            IN CASE IT'S TOO LONG
        BRCLR   SCSR,X $20 *    WAIT FOR RDRF
        LDAB    SCDAT,X         LENGTH OF PROG (LO BYTE)
        STD     TOC1,X          SAVE LENGTH
* THEN DOWNLOAD PROGRAM
        LDY     #$0000          INIT POINTER
* READ IN PROGRAM AND PUT INTO RAM
BK2     EQU     *
        CPY     TOC1,X          UNTIL LENGTH IS REACHED
        BEQ     STAR
        BRCLR   SCSR,X $20 *    WAIT FOR RDRF
        LDAA    SCDAT,X
        STAA    $00,Y
        STAA    SCDAT,X         HANDSHAKE
        INY
        BRA     BK2
************************************************
* ALL START USER'S PROGRAM
*
STAR    EQU     *
        JMP     $0000
************************************************
* MASK I.D. #
        FDB     $0000           ($BFD2)
************************************************
* EEPROM ERASE SUBROUTINE
*
* ASSUMES CALLING ROUTINE HAS ALREADY SET ERASE
* AND EELAT BITS, AS WELL AS ACCESSED WHATEVER
* IS TO BE ERASED
*
* ENTRY X=$1000
* EXIT X=$1000, Y=$0000
*
ERASE   EQU     *
* SET EEPGM BIT
        BSET    PPROG,X $01
* 10 MILLISEC DELAY @ 2.1 MHZ
        LDY     #3000
BK1     DEY
        BNE     BK1
* TURN OFF ERASE AND EELAT BITS
        CLR     PPROG,X
        RTS
************************************************
* VECTORS (FIRST SIX DON'T POINT TO RAM)
*
*        ORG     $BFD6           NEEDED IF BOOTROM < MAX
*
*        FDB     $100-60         SCI
*        FDB     $100-57         SPI
*        FDB     $100-54         PULSE ACCUM INPUT EDGE
*        FDB     $100-51         PULSE ACCUM OVERFLOW
*        FDB     $100-48         TIMER OVERFLOW
*        FDB     $100-45         TIMER OUTPUT COMPARE 5
        FDB     $100-42         TIMER OUTPUT COMPARE 4
        FDB     $100-39         TIMER OUTPUT COMPARE 3
        FDB     $100-36         TIMER OUTPUT COMPARE 2
        FDB     $100-33         TIMER OUTPUT COMPARE 1
        FDB     $100-30         TIMER INPUT CAPTURE 3
        FDB     $100-27         TIMER INPUT CAPTURE 2
        FDB     $100-24         TIMER INPUT CAPTURE 1
        FDB     $100-21         REAL TIME INT
        FDB     $100-18         IRQ
        FDB     $100-15         XIRQ
        FDB     $100-12         SWI
        FDB     $100-9          ILLEGAL OP-CODE
        FDB     $100-6          COP FAIL
        FDB     $100-3          CLOCK MONITOR
        FDB     #BEGIN          RESET
        END
